<?php
/*
Plugin Name: 	Advanced Activity Widget
Plugin URI: 	http://codecanyon.net/item/advanced-activity-widget/1525794
Description:	Displays an advanced tabbed interface to display your WordPress latest comments, most recent posts, most popular posts of all time and WordPress tag cloud.
Version: 		2.0
Author: 		20Script
Author URI: 	http://www.20script.ir
License: 		Sold exclusively on CodeCanyon
*/

if(!class_exists('AdvancedActivityWidget') and !class_exists('AdvancedActivityWidget_Widget')){
	
	class AdvancedActivityWidget{
		
		/**
		 * The plugins slug
		 */
		const SLUG = 'advancedactivitywidget';
		
		/**
		 * The plugins current version number
		 */
		const VERSION = '2.0';
		
		protected $pluginPath;  
    	protected $pluginUrl;
		
		/**
		 * Create an instance of the plugin
		 */
		public function __construct(){
					
			/* Register (de)activation hook */
			register_activation_hook(__FILE__, array($this, 'activatePlugin'));
			register_deactivation_hook(__FILE__, array($this, 'deactivatePlugin'));
			
			/* Register Widget */
			add_action('widgets_init', array($this, 'registerWidget'));
			
			/* Register and load textdomain */
			load_plugin_textdomain(self::SLUG, null, basename(dirname(__FILE__)) . '/languages/');
			
			/* Register Widget Stylesheets */
			if(is_admin()){
				
				/* Register additional styles for "widgets" admin page */
				add_action('admin_print_styles-widgets.php', array($this, 'registerAdminStyle'));
				
			} else if(is_active_widget(false, false, self::SLUG, true)){
				
				/* Register AdvancedPostWidget widget styles */
				add_action('wp_print_styles', array($this, 'registerStyle'));
				
			}		
			add_action('wp_enqueue_scripts', array($this, 'add_javascripts'));	
			
			/* Track the post views */
			add_action('wp_head', array($this, 'track_postviews'));
			
			// Add the shortcode
			add_shortcode('AAW_Views', array($this, 'aaw_shortcode'));
			
			// Add shortcode support for widgets  
    		add_filter('widget_text', 'do_shortcode'); 
		}
		
		/**
		 * Track the actual post views
		 */
		public function track_postviews() {
		   if(is_single()) {
			  global $post;
			  $current_views = get_post_meta($post->ID, "aaw_views", true);
			  if(!isset($current_views) OR empty($current_views) OR !is_numeric($current_views) ) {
				 $current_views = 0;
			  }
			  $new_views = $current_views + 1;
			  update_post_meta($post->ID, "aaw_views", $new_views);
		   }
		}
		
		/**
		 * Make sure jQuery is included in plugin and include javascript file(s)
		 */
		public function add_javascripts() {
    		wp_enqueue_script('jquery');
			wp_enqueue_script('advanced-activity-widget', WP_PLUGIN_URL.'/advanced-activity-widget/js/advanced-activity-widget.js', '', '', false);
			wp_enqueue_script('aaw-jquery-ui', WP_PLUGIN_URL.'/advanced-activity-widget/js/jquery-ui-1-5-2-min.js', '', '', false);
		}    
		
		/**
		 * Register options and create database tables
		 *
		 * Will be run through register_activation_hook()
		 */
		public function activatePlugin(){

		}
		
		/**
		 * Deregister options and delete database tables
		 *
		 * Will be run through register_deactivation_hook
		 */
		public function deactivatePlugin(){

		}
		
		/**
		 * Register the Widget
		 *
		 * Will be run within "widgets_init" action
		 */
		public function registerWidget(){
			
			register_widget('AdvancedActivityWidget_Widget');
			
		}
		
		/**
		 * Register the widgets admin stylesheet
		 *
		 * Will be run in "admin_print_styles-widgets.php" action and only on widgets admin page
		 */
		public function registerAdminStyle(){
			
			/* register Style */
			wp_register_style(self::SLUG . '-admin', plugins_url('css/advanced-activity-widget-admin.css', __FILE__), array(), self::VERSION, 'screen');
						
			/* Enqueue Style */
			wp_enqueue_style(self::SLUG . '-admin');
			
		}
		
		/**
		 * Register the widgets stylesheet
		 *
		 * Will be run in "wp_print_styles" action and only on frontend pages and if widget is actually used
		 */
		public function registerStyle(){
			
			/* Register style */
			wp_register_style(self::SLUG, plugins_url('css/advanced-activity-widget.css', __FILE__), array(), self::VERSION, 'screen');
			
			/* Enqueue style */
			wp_enqueue_style(self::SLUG);
			
		}
		
		/**
		* Retrieve the number of views for a certain post
		*
		* Finds the current views for a post, returning 0 if there are none
		*
		* @global object $post The post object
		* @return integer $current_views The number of views the post has
		*
		*/
		function retrieve_view_count() {
			global $post;
			$current_views = get_post_meta($post->ID, "aaw_views", true);
			if(!isset($current_views) OR empty($current_views) OR !is_numeric($current_views) ) {
			  $current_views = 0;
		}
		
		return $current_views;
		}
		
		/**
		 * Create a shortcode for the plugin
		 */
		public function aaw_shortcode(){  
    
			return $this->retrieve_view_count(); 
		}  


		/**
		 * Retrieves the most viewed posts
		 *
		 * Shows a list of posts ordered by their view count
		 *
		 * @param integer $limit - The number of posts to display
		 * @param integer $chars - The number of characters to display for the post body
		 *
		 */
		public function aaw_get_most_viewed_posts($limit = 5, $chars = 35, $size = 50, $displayMostIcon = true) {
			$post_views = '';
			$post_title = '';
					
		   $args = array(
			  "posts_per_page" => $limit,
			  "post_type" => "post",
			  "post_status" => "publish",
			  "orderby" => "meta_value_num",
			  "order" => "DESC"
		   );

		   $aaw_get_most_viewed = new WP_Query($args);
			
		   if($aaw_get_most_viewed->have_posts()){
			   while ( $aaw_get_most_viewed->have_posts() ) : $aaw_get_most_viewed->the_post();
					 
					$post_views = number_format_i18n(intval(self::retrieve_view_count()));
					$post_title = get_the_title();
					
					if($chars > 0) {
						$post_title = self::snippet_text($post_title, $chars);
					}
	
					echo '<li>';
						if($displayMostIcon){
							echo self::featured_image_thumb($size);
						}
						echo '<div class="info">';
						echo '<a title="'.get_the_title().'" href="'.get_permalink().'">'.$post_title.'</a>';
						echo '<span class="meta">'.$post_views.' '.__('views', self::SLUG).'</span>';
						echo '</div>'; 
					echo '</li>'."\n";
					
			   endwhile;
			} else {
				echo '<li>'.__('No posts available', self::SLUG).'</li>'."\n";
			}
		
		    // Reset Post Data
		    wp_reset_postdata();
		
		}
		
		/**
		 * Retrieves the latest posts
		 *
		 * Shows a list of latest posts ordered by the date added
		 *
		 * @param integer $limit - The number of posts to display
		 * @param integer $chars - The number of characters to display for the post body
		 *
		 */
		public function aaw_get_latest_posts($limit = 5, $chars = 35, $size = 50, $displayLatestIcon = true) {
			global $wpdb, $post;
			$post_title = '';
			$post_date = '';
			$post_time = '';
			
			$aaw_get_latest = new WP_Query('showposts='. $limit .'&orderby=post_date&order=desc');	
			
			if($aaw_get_latest->have_posts()){
				while($aaw_get_latest->have_posts()) : $aaw_get_latest->the_post(); 
					
					$post_title = get_the_title();
					$post_date = get_the_date('j M Y');
					$post_time = get_the_time('g:i a');
					
					if($chars > 0) {
						$post_title = self::snippet_text($post_title, $chars);
					}
						
					echo '<li>';
						if($displayLatestIcon){
							echo self::featured_image_thumb($size);
						}
						echo '<div class="info">';
						echo '<a title="'.get_the_title().'" href="'.get_permalink().'">'.$post_title.'</a>';
						echo '<span class="meta">'.$post_date.' '.__('at', self::SLUG).' '.$post_time.'</span>';
						echo '</div>'; 
					echo '</li>'."\n";
					
				endwhile; 
			} else {
				echo '<li>'.__('No posts available', self::SLUG).'</li>'."\n";
			}
			
			 // Reset Post Data
		    wp_reset_postdata();
		}
		
		/**
		 * Retrieves the latest comments
		 *
		 * Shows a list of latest comments ordered by the date added
		 *
		 * @param integer $limit - The number of posts to display
		 * @param integer $chars - The number of characters to display for the post body
		 *
		 */
		 public function aaw_get_latest_comments($limit = 5, $chars = 35, $size = 50, $displayCommentsIcon = true) {
			global $wpdb;
			
			$commentsql = "SELECT DISTINCT ID, post_title, post_password, comment_ID, comment_post_ID, comment_author, comment_author_email, comment_date_gmt, comment_approved, comment_type,comment_author_url,
			SUBSTRING(comment_content,1,".$chars.") AS com_excerpt
			FROM $wpdb->comments
			LEFT OUTER JOIN $wpdb->posts ON ($wpdb->comments.comment_post_ID =
			$wpdb->posts.ID)
			WHERE comment_approved = '1' AND comment_type = '' AND
			post_password = ''
			ORDER BY comment_date_gmt DESC LIMIT ".$limit;
			
			$comments = $wpdb->get_results($commentsql);
			
			if($comments){
				foreach($comments as $comment) {
					
					echo '<li>';
						if($displayCommentsIcon){
							echo '<a href="'.get_permalink($comment->ID).'#comment-'.$comment->comment_ID.'" title="'. __('Commented on: ', self::SLUG). $comment->post_title.'">';
							echo get_avatar($comment, $size);
							echo '</a>';
						}
						echo '<div class="info">';
						echo '<a href="'.get_permalink($comment->ID).'#comment-'.$comment->comment_ID.'" title="'. __('Commented on: ', self::SLUG). $comment->post_title.'">';
							echo '<i>'.strip_tags($comment->comment_author).'</i>: '.strip_tags($comment->com_excerpt).'...';
						echo '</a>';
						echo '<span class="meta">'.get_comment_date('j M Y',$comment->comment_ID).' '.__('at', self::SLUG).' '.get_comment_date('g:i a',$comment->comment_ID).'</span>';
						echo '</div>';
					echo '</li>';
	
				}
			} else {
				echo '<li>'.__('No comments available', self::SLUG).'</li>'."\n";
			}
		}
		 
		
		/**
		 * Shortens a string of text
		 *
		 * Takes the input string and returns a shortened version of the string
		 *
		 * @param string $text - The text string to shorten
		 * @param integer $length - The number of characters to strip th etext down to
		 *
		 */
		private function snippet_text($text, $length = 0) {
			if(defined('MB_OVERLOAD_STRING')) {
			  $text = @html_entity_decode($text, ENT_QUOTES, get_option('blog_charset'));
				if (mb_strlen($text) > $length) {
					return htmlentities(mb_substr($text,0,$length), ENT_COMPAT, get_option('blog_charset')).'...';
				} else {
					return htmlentities($text, ENT_COMPAT, get_option('blog_charset'));
				}
			} else {
				$text = @html_entity_decode($text, ENT_QUOTES, get_option('blog_charset'));
				if (strlen($text) > $length) {
					return htmlentities(substr($text,0,$length), ENT_COMPAT, get_option('blog_charset')).'...';
				} else {
					return htmlentities($text, ENT_COMPAT, get_option('blog_charset'));
				}
			}
		}
		
		/**
		 * Retrieves the image for a post
		 *
		 * Uses the post_thumbnails if available or
		 * searches through the post and retrieves the first found image for use as thumbnails
		 */
		function featured_image_thumb($size = 50) {
			global $post;
			// If a featured image has been set, use the featured-thumbnail size that was set above with the class of 'thumb'
			if (has_post_thumbnail() ) {
				echo '<a href="'.get_permalink().'" title="'.get_the_title().'" >';
				the_post_thumbnail(array($size,$size),array('class' => 'thumb'));
				echo '</a>';
			}
			// If a featured image is not set, get the first image in the post content
			else {
				$first_img = '';
				ob_start();
				ob_end_clean();
				$output = preg_match_all('/<img.+src=[\'"]([^\'"]+)[\'"].*>/i', $post->post_content, $matches);
				$first_img = $matches[1][0];
		
				// Define a default fallback image in case a featured image is not set and there are no images in the post content
				if(empty($first_img)){
					$first_img = WP_PLUGIN_URL.'/advanced-activity-widget/images/nothumb.png';
				}
		
				// Generate the HTML code to display the image and resize the image with timthumb.php
				return '<a title="'.get_the_title().'" href="'.get_permalink().'"><img class="thumb" src="'.WP_PLUGIN_URL.'/advanced-activity-widget/lib/timthumb.php?src=' . $first_img .'&w='.$size.'&h='.$size.'" alt="" /></a>';
			}
		}
			
	}
		
	class AdvancedActivityWidget_Widget extends WP_Widget{
		
		/**
		 * The Widgets slug
		 */
		const SLUG = 'advancedactivitywidget';
		
		/**
		 * Create a widget instance and set the base infos
		 */
		public function __construct(){
			
			/* Widget settings */
			$widgetOptions = array(
				'classname' => self::SLUG,
				'description' => __('Displays an Advanced Tabbed Activity Widget', self::SLUG)
			);
			
			/* Widget control settings */
			$controlOptions = array(
				'id_base' => self::SLUG,
				'width' => 280
			);
			
			/* Create the widget */
			parent::__construct(self::SLUG, 'Advanced Activity Widget', $widgetOptions, $controlOptions);
			
		}
		
		/**
		 * Display the actual Advanced Social Widget
		 *
		 * @param Array $args
		 * @param Array $instance
		 */
		public function widget($args, $instance){
			
			extract($args);
			
			/* Get options and apply variables */
			$widgetWidth = $instance['widgetwidth'];
			
			$showpopular  = $instance['showpopular'];
			$popularAmount = $instance['popular_amount'];
			$popularChars = $instance['popular_chars'];
			$popularIconSize = $instance['popular_icon_size'];
			$displayPopularIcon = $instance['display_popular_icon'];
			
			$showrecent = $instance['showrecent'];
			$latestAmount = $instance['latest_amount'];
			$latestChars = $instance['latest_chars'];
			$latestIconSize = $instance['latest_icon_size'];
			$displayLatestIcon = $instance['display_latest_icon'];
			
			$showcomments = $instance['showcomments'];
			$commentsAmount = $instance['comments_amount'];
			$commentChars = $instance['comments_chars'];
			$commentsIconSize = $instance['comments_icon_size'];
			$displayCommentsIcon = $instance['display_comments_icon'];
			
			$showtags = $instance['showtags'];
			$tags_amount = $instance['tags_amount'];
			
			/* Before Widget HTML */
			echo $before_widget;
				
			/* Advanced Social Widget */
			$aaw = new AdvancedActivityWidget($instance);
			include 'includes/widget.php';
			
			/* After Widget HTML */
			echo $after_widget;
			
		}
		
		/**
		 * Update Widget settings and refresh data for this Widget
		 *
		 * @param Array $newInstance
		 * @param Array $oldInstance
		 * @return Array
		 */
		public function update($newInstance, $oldInstance){
			
			/* Update widget settings */
			$instance = $oldInstance;
			$instance['widgetwidth'] = ((is_numeric($newInstance['widgetwidth'])) ? trim($newInstance['widgetwidth']) : 250);
			
			$instance['showpopular'] = $newInstance['showpopular'];
			$instance['display_popular_icon'] = $newInstance['display_popular_icon'];
			$instance['popular_icon_size'] = ((is_numeric($newInstance['popular_icon_size'])) ? trim($newInstance['popular_icon_size']) : 50);
			$instance['popular_amount'] = ((is_numeric($newInstance['popular_amount'])) ? trim($newInstance['popular_amount']) : 5);
			$instance['popular_chars'] = ((is_numeric($newInstance['popular_chars'])) ? trim($newInstance['popular_chars']) : 35);
			
			$instance['showrecent'] = $newInstance['showrecent'];
			$instance['display_latest_icon'] = $newInstance['display_latest_icon'];
			$instance['latest_icon_size'] = ((is_numeric($newInstance['latest_icon_size'])) ? trim($newInstance['latest_icon_size']) : 50);
			$instance['latest_amount'] = ((is_numeric($newInstance['latest_amount'])) ? trim($newInstance['latest_amount']) : 5);
			$instance['latest_chars'] = ((is_numeric($newInstance['latest_chars'])) ? trim($newInstance['latest_chars']) : 35);
			
			$instance['showcomments'] = $newInstance['showcomments'];
			$instance['display_comments_icon'] = $newInstance['display_comments_icon'];
			$instance['comments_icon_size'] = ((is_numeric($newInstance['comments_icon_size'])) ? trim($newInstance['comments_icon_size']) : 50);
			$instance['comments_amount'] = ((is_numeric($newInstance['comments_amount'])) ? trim($newInstance['comments_amount']) : 5);
			$instance['comments_chars'] = ((is_numeric($newInstance['comments_chars'])) ? trim($newInstance['comments_chars']) : 35);
			
			$instance['showtags'] = $newInstance['showtags'];
			$instance['tags_amount'] = ((is_numeric($newInstance['tags_amount'])) ? trim($newInstance['tags_amount']) : 25);
			
			return $instance;
			
		}
		
		/**
		 * Show the Widgets settings form
		 *
		 * @param Array $instance
		 */
		public function form($instance){
			
			/* Apply default values */
			$defaults = array(
				'widgetwidth'           => '250',
				'showpopular' 			=> true,
				'display_popular_icon'  => true,
				'popular_icon_size'     => '50',
				'popular_amount'		=> '5',
				'popular_chars'			=> '35',
				'showrecent' 			=> true,
				'display_latest_icon'   => true,
				'latest_icon_size'      => '50',
				'latest_amount'			=> '5',
				'latest_chars'			=> '35',
				'showcomments' 			=> true,
				'display_comments_icon' => true,
				'comments_icon_size'    => '50',
				'comments_amount'		=> '5',
				'comments_chars'		=> '35',
				'showtags' 				=> true,
				'tags_amount'			=> '25'
			);
			$instance = wp_parse_args((array) $instance, $defaults);
			
			/* Show Advanced Social Widget Options Form */
			include 'includes/form.php';
			
		}
		
	}
	
}

/* Create instance of AdvancedSocialWidget */
if(class_exists('AdvancedActivityWidget')){
	$AdvancedActivityWidget = new AdvancedActivityWidget();
}